@@ -5,3 +5,8 @@ dim.tbl_spark <- function(x) {
   columns <- invoke(sdf, "columns")
   c(rows, length(columns))
 }
+
+#' @export
+type_sum.spark_jobj <- function(x) {
+  paste0(jobj_info(x)$repr)
+}
